#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  9 14:00:13 2020

@author: marisaeisenberg
"""

import scipy.optimize as optimize
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as stats
from scipy.stats import poisson
from scipy.stats import norm
from scipy.integrate import odeint as ode

# Generate some data to fit
data = norm.rvs(size=20,loc=2,scale=1)
plt.hist(data)

# Define the likelihood
def likelihood(data,mu):
    # each data point is independent (so we can product them to get the overall probability)
    # the probability density of observing that data given a particular mu is norm.pdf(data,mu,1)
    return(np.prod(norm.pdf(data,mu,1)) )  

# Define the prior (chosen arbitrarily in this case)
def prior(mu):
    # returns the density for a given possible value of mu, supposing a normal prior with 
    # mean 1, and SD = 1.5
    return(norm.pdf(mu,1,1.5)) 


# Set up the MCMC

# Define the proposal distribution
def proposaldist(paramval):
    # we'll propose new points as a normal distribution centered on the current point
    return(norm.rvs(size=1,loc=paramval,scale=0.5) )

# Set up parameters
mucurr = 5 #start value for mu
numsteps = 5000 # number of steps to run
samples = np.zeros(numsteps) # we will store the sampled mu values here

# Run the MCMC!
for i in range(numsteps):
  samples[i] = mucurr
  munext = proposaldist(mucurr)
  
  probcurr = likelihood(data,mucurr)*prior(mucurr)
  probnext = likelihood(data,munext)*prior(munext)
  
  acceptratio = probnext/probcurr
  
  if acceptratio >= 1:
    mucurr = munext
  elif np.random.uniform()<acceptratio:
    mucurr = munext

# Plot the results
    
# Plot the trace of mu
plt.figure()
plt.plot(range(numsteps),samples)


# Plot the distribution of our estimates of mu, compare to likelihood and prior
# take just the later part of the chain, after burn-in
posteriorsample = samples[500:]

plt.figure()
plt.hist(posteriorsample, normed=True)



